%{

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <cjson/cJSON.h>


extern int yylex();
extern int yyparse();
extern FILE* yyin;
extern char*  yytext;

int yydebug=1;

void yyerror(const char* s);

char* catstr(char* s1, char* s2);
cJSON* jroot;
%}

%union {
	int iv;
	double fv;
	char* sv;
	cJSON* jv;
	int subtok;
}

%token IDENTIFIER 

%token SELECT FROM WHERE INSERT INTO VALUES DELETE UPDATE SET JOIN CREATE TABLE 
%token AND OR NOT IN 
%token INT FLOAT STRING
%token LPAREN RPAREN COMMA
%token PRIMARY_KEY

%token QUIT NEWLINE

%type <sv> IDENTIFIER data_type PRIMARY_KEY col_options
%type <jv> create_definition create_col_list create_table_stmt

%start statement

%%

statement: NEWLINE
	| QUIT {printf("bye!\n"); exit(0); }
	| sql_statement 
;

sql_statement: create_table_stmt 

create_table_stmt: CREATE TABLE IDENTIFIER NEWLINE
	{
		cJSON* node = cJSON_CreateObject();
		cJSON_AddStringToObject(node, "type", "create_table");
		cJSON_AddStringToObject(node, "table_name", $3);
		cJSON_AddItemToArray(jroot, node);
	}

create_table_stmt: CREATE TABLE IDENTIFIER '(' create_col_list ')' NEWLINE {
		cJSON* node = cJSON_CreateObject();
		cJSON_AddStringToObject(node, "type", "create_table");
		cJSON_AddStringToObject(node, "table_name", $3);
		cJSON_AddItemToObject(node, "cols", $5);
		cJSON_AddItemToArray(jroot, node);
		$$=node;
};

create_col_list: create_definition {
	cJSON* node = cJSON_CreateArray();
	cJSON_AddItemToArray(node, $1);
	$$=node;
	// printf("得create_definition %s\n", cJSON_Print($1));
}
    |  create_col_list ',' create_definition {
		// printf("迭代 create_col_list\n %s", cJSON_Print($1));
		// printf("迭代 create_definition\n %s", cJSON_Print($3));

		cJSON_AddItemToArray($1, $3);

		$$=$1;
		// printf("迭代\n %s", cJSON_Print($$));
	}
    ;

create_definition: IDENTIFIER data_type col_options {
		printf("得列 %s %s with options %s: %d\n", $1, $2, $3, strlen($3));

		cJSON* node = cJSON_CreateObject();
		cJSON_AddStringToObject(node, "type", "create_column");
		cJSON_AddStringToObject(node, "column_name", $1);
		cJSON_AddStringToObject(node, "data_type", $2);
		if (strlen($3) > 0){
			cJSON_AddTrueToObject(node, "primary_key");
		} else {
			cJSON_AddFalseToObject(node, "primary_key");
		}
		$$ = node;
};
col_options: {$$ = "";}
	| PRIMARY_KEY {
		// printf("得到主键 %s\n", $1);
		$$ = $1;
	}

data_type: INT
	| FLOAT
	| STRING
;
%%

int main(int ac, char** av) {
	jroot = cJSON_CreateArray();

	if (ac != 2) {
		printf("Usage: %s <sql>\n", av[0]);
		exit(1);
	}

	// yyin = av[1];
	yy_scan_string(av[1]);
	int res = yyparse();
	printf("%s\n", cJSON_Print(jroot));
	return res;
}

void yyerror(const char *s) {
	printf("error: %s at %s\n", s, yytext);
	exit(1);
}

char* catstr(char* s1, char* s2) {
	char* s = malloc(strlen(s1)+strlen(s2)+1);
	strcpy(s, s1);
	strcat(s, s2);
	return s;
}

